@@ -120,6 +120,10 @@ sub connect {
     my $new_arg = *$self->{ssl_new_arg};
     $arg->{SSL_Debug} = $debug;
 
+    # setup SNI if available
+    $ssl->can("set_tlsext_host_name") and
+        $ssl->set_tlsext_host_name(*$self->{ssl_peer_addr});
+
     eval {
         local $SIG{ALRM} = sub { $self->die_with_error("SSL connect timeout") };
         # timeout / 2 because we have 3 possible connects here
@@ -194,6 +198,12 @@ sub get_cipher {
     *$self->{ssl_ssl}->get_cipher(@_);
 }
 
+sub pending {
+    my $self = shift;
+    $self = $REAL{$self} || $self;
+    *$self->{ssl_ssl}->pending(@_);
+}
+
 sub ssl_context {
     my $self = shift;
     $self = $REAL{$self} || $self;
@@ -416,6 +426,11 @@ sub proxy {
     }
 
     $proxy_server =~ s|\Ahttps?://||i;
+    # sanitize the end of the string too
+    # see also http://www.nntp.perl.org/group/perl.libwww/2012/10/msg7629.html
+    # and https://github.com/nanis/Crypt-SSLeay/pull/1
+    # Thank you Mark Allen and YigangX Wen
+    $proxy_server =~ s|(:[1-9][0-9]{0,4})/\z|$1|;
     $proxy_server;
 }
 
@@ -518,6 +533,11 @@ Attempts to read up to 32KiB of data from the socket. Returns
 C<undef> if nothing was read, otherwise returns the data as
 a scalar.
 
+=item pending
+
+Provides access to OpenSSL's C<pending> attribute on the SSL connection
+object.
+
 =item getline
 
 Reads one character at a time until a newline is encountered,
